{-----------------------------------------------------------------------------

 CF_File : file interface definition and implementation of the game ChiFu.

 Consists of:

 Types:
 -TFile : standard file IO interface definition
   Routines:
   -Init        : initializes the interface
   -ReadScores  : reads the hiscore list from file
   -WriteScores : writes the hiscore list to file

 Constants:
 -ScoreFileName : name of the hiscore list file

 Variables:
 -ScoreFIle : the hiscore list file interface

 Routines:

 Defaults:
 -Initializes the ScoreFile and reads the hiscore list

-----------------------------------------------------------------------------}
unit CF_File;

interface

uses
  CF_Set,
  CF_Scor;

const
  { The file name of the score file }
  ScoreFileName : string [9] = 'ChiFu.Scr';

type
  { The file interface definition }
  TFile = object
            public
              { Initializes the file interface }
              constructor Init;

              { Reads the hiscores from file }
              procedure ReadScores;

              { Writes the hiscores to file }
              procedure WriteScores;

            private
              { The actual file }
              ScoreFile : file of TEntry;

              { Checks on the score file. If not present, makes a new one }
              procedure CheckFile;

          end; { object }

var
  ScoreFile : TFile;

implementation

{ Initializes the file interface }
constructor TFile.Init;

begin
  Assign (ScoreFile, ScoreFileName);
  CheckFile;
end; { Init }

{$I-}

{ Checks on the score file. If not present, makes a new one }
procedure TFile.CheckFile;

var
  PlayMode : TPlayMode;

begin
  { Look if the file exists }
  Reset (ScoreFile);
  if IOResult <> 0 then begin
    { File didn't exist -> make new one }
    for PlayMode := Small to Large do begin
      Scores [PlayMode].ClearHiScores;
    end; { for }

    WriteScores;
  end; { then }
end; { CheckFile }

{$I+}

{ Reads the hiscores from file }
procedure TFile.ReadScores;

var
  PlayMode : TPlayMode;
  EntryNr  : byte;

begin
  { Set the file for reading }
  Reset (ScoreFile);

  { And for all play modes }
  for PlayMode := Small to Large do begin
    { Read all entries }
    for EntryNr := 0 to MaxScores do begin
      Read (ScoreFile, Scores [PlayMode].HiScores [EntryNr]);
    end; { for }
  end; { for }

  Close (ScoreFile);
end; { ReadScores }

{ Writes the hiscores to file }
procedure TFile.WriteScores;

var
  PlayMode : TPlayMode;
  EntryNr  : byte;

begin
  { Set the file for writing }
  Rewrite (ScoreFile);

  { And for all play modes }
  for PlayMode := Small to Large do begin
    { Write all entries }
    for EntryNr := 0 to MaxScores do begin
      Write (ScoreFile, Scores [PlayMode].HiScores [EntryNr]);
    end; { for }
  end; { for }

  Close (ScoreFile);
end; { WriteScores }

begin
  ScoreFile.Init;
  ScoreFile.ReadScores;
end. { CF_File }