{-----------------------------------------------------------------------------

 CF_Pict : picture definitions of the game ChiFu.

 Consists of:

 Types:
 -TArrow    : all sorts of arrows possible
 -TPictPtr  : standard picture definition
 -TPictFile : standard picture file definition

 Constants:
 -StoneSize   : size of the stones (in X and Y) in pixels
 -LetterHeight,
 -LetterWidth : letter height and width in pixels

 Variables:
 -StonePict : all stone pictures
 -CharPict  : all character (letter) pictures
 -ArrowPict : all arrow pictures
 -TitlePict : the title screen picture

 Routines:

 Defaults:
 -loads pictures and shows title screen

-----------------------------------------------------------------------------}
unit CF_Pict;

interface

uses
  CF_Set;

const
  { Standard stone size }
  StoneSize = 20;
  LetterHeight = 12;
  LetterWidth = 12;

type
  { All sorts of arrows }
  TArrow = (LeftAr, RightAr);

  { Picture pointer reference }
  TPictPtr  = ^pointer;
  TPictFile = file of byte;

var
  { Stone pictures }
  StonePict : array [0..AmDiffStones] of TPictPtr;
  CharPict  : array [0..96] of TPictPtr;
  TitlePict : TPictPtr;

implementation

uses
  CF_Grph;

var
  CharNr,
  VarietyNr  : word;
  NumberText : string;
  PictFile   : TPictFile;

{ Reads the picture from file in FileName to the Picture; size Size }
procedure ReadPict (var Picture : TPictPtr; Size : word);

var
  ElemNr   : word;
  PictPtr  : TPictPtr;

  { Increases the BytePtr }
  procedure IncPtr;

  var
    SegPtr,
    OfsPtr : word;

  begin
    SegPtr := Seg (PictPtr^);
    OfsPtr := Ofs (PictPtr^);
    Inc (Ofsptr);
    if OfsPtr = 0 then begin
      Inc (SegPtr);
    end; { then }
    PictPtr := Ptr (SegPtr, OfsPtr);
  end; { IncPtr }

begin
  GetMem (Picture, Size + 1);
  PictPtr := Picture;
  for ElemNr := 1 to Size do begin
    Read (PictFile, byte (PictPtr^));
    IncPtr;
  end; { for }
end; { ReadPict }

begin
  { Read the title sequence }
  Assign (PictFile, 'ChiFu.ttl');
  Reset (PictFile);
  ReadPict (TitlePict, 24775);
  Close (PictFile);

  { Read the stone pictures }
  for VarietyNr := 1 to AmDiffStones do begin
    Str (VarietyNr, NumberText);
    Assign (PictFile, 'ChiFu.p' + NumberText);
    Reset (PictFile);
    ReadPict (StonePict [VarietyNr], 235);
    Close (PictFile);
  end; { for }

  { And show the title sequence }
  Screen.ShowTitleScreen;

  { And all characters and arrows }
  Assign (PictFile, 'ChiFu.chr');
  Reset (PictFile);
  for CharNr := 0 to 96 do begin
    ReadPict (CharPict [CharNr], 95);
  end; { for }
  Close (PictFile);
end. { CF_Pict }