{-----------------------------------------------------------------------------

 CF_Set : all game settings and options of the game ChiFu.

 Consists of:

 Types:
 -TOption   : all possible standard options in game
 -TSetting  : all settings in game
 -TPlayMode : all possible play modes

 Constants:
 -StdSettings   : all settings for all play modes
 -AmDiffStones  : amount of different stones in game
 -PlayModeNames : the names of all play modes

 Variables:
 -Settings : chosen settings of the game
 -PlayMode : chosen playmode of the game
 -SoundOn  : whether the sound is on or not
 -FullRows : the amount of full rows at beginning of game

 Routines:

 Defaults:
 -initializes all the settings to normal

-----------------------------------------------------------------------------}
unit CF_Set;

interface

type
  { All possible standard options in game }
  TOption = (PlayGame, ChangeFullRows, SoundOnOff, ChangePlayMode, ShowHi, Instructions, ExitGame);

  { All settings in game }
  TSetting = (FieldLength,    { Field size in Y }
              FieldWidth,     { Field size in X }
              BlockLength,    { Actual block length }
              RowLength,      { Min. amount of stones per row to take away }
              Joker,          { Whether or not to use the joker }
              AmVarieties);   { Amount of different stones }

  { All possible play modes }
  TPlayMode = (Small, Normal, Large);

  { A single setting-list element }
  TSettingList = array [TSetting] of byte;

const
  { The standard settings for all play modes }
  StdSettings : array [TPlayMode] of TSettingList =
                ((17,  3, 3, 3, 0, 4),
                 (17,  7, 3, 3, 0, 6),
                 (17, 11, 3, 3, 0, 7));

  { Amount of different stones in game }
  AmDiffStones = 7;

  { And the names of all play modes }
  PlayModeNames : array [TPlayMode] of string [6] =
                  ('Small', 'Normal', 'Large');

var
  Settings : TSettingList;
  PlayMode : TPlayMode;
  SoundOn  : boolean;
  FullRows : byte;

implementation

begin
  { Take normal play as default }
  PlayMode := Normal;
  Settings := StdSettings [PlayMode];

  { With no full rows }
  FullRows := 0;

  { And sound on by default }
  SoundOn := true;

end. { CF_Set }