{ Enables user to make new stone pictures }
program MakePict;

uses
  Graph,
  WinDos,
  Strings,
  Crt;

type
  TPictPtr = ^pointer;

const
  StoneSize = 19;
  PictSize  = 235;

var
  PictureSmall : TPictPtr;
  Picture      : array [1..StoneSize, 1..StoneSize] of byte;
  Changed,
  Finished     : boolean;
  UserChoice   : char;











{ Graphical part ------------------------------------------------------------}

const
  SmallOffsetX   = 300;
  SmallOffsetY   = 150;
  ColorOffsetX   = 300;
  ColorOffsetY   = 18;
  SingleElemSize = 8;

{ Initializes the screen into graphics-mode }
procedure InitializeScreen;

var
  Driver,
  Mode : integer;

begin
  { Put screen in graphics-mode }
  Driver := Detect;
  InitGraph (Driver, Mode, '.');
end; { Initialize }

{ Shows the picture on screen (large and small) }
procedure ShowPicture;

var
  PrevX,
  PrevY,
  X,
  Y : byte;

begin
  { Put the picture on screen; the large one }
  for Y := 1 to StoneSize do begin
    for X := 1 to StoneSize do begin
      SetFillStyle (1, Picture [X, Y]);
      Bar (X * SingleElemSize, Y * SingleElemSize,
           (X + 1) * SingleElemSize - 1, (Y + 1) * SingleElemSize - 1);
    end; { for }
  end; { for }

  { And the small one }
  PutImage (SmallOffsetX, SmallOffsetY, PictureSmall^, NormalPut);
end; { ShowPicture }

{ Shows the cursor and deletes the previous one }
procedure ShowCursor (var X, Y, PrevX, PrevY : byte);

begin
  { Delete previous one }
  SetColor (Picture [PrevX, PrevY]);
  Rectangle (PrevX * SingleElemSize + 1, PrevY * SingleElemSize + 1,
             (PrevX + 1) * SingleElemSize - 2, (PrevY + 1) * SingleElemSize - 2);

  { And show the new one }
  if Picture [X, Y] < 10 then begin
    SetColor (15);
  end { then }
  else begin
    SetColor (0);
  end; { else }
  Rectangle (X * SingleElemSize + 1, Y * SingleElemSize + 1,
             (X + 1) * SingleElemSize - 2, (Y + 1) * SingleElemSize - 2);
end; { ShowCursor }

{ Shows the current color in use }
procedure ShowColor (Color : byte);

begin
  { Show the color in use }
  SetFillStyle (1, Color);
  Bar (ColorOffsetX, ColorOffsetY,
       ColorOffsetX + SingleElemSize, ColorOffsetY + SingleElemSize);

  { And the edge, if needed }
  if Color = 0 then begin
    SetColor (15);
    Rectangle (ColorOffsetX, ColorOffsetY,
               ColorOffsetX + SingleElemSize, ColorOffsetY + SingleElemSize);
  end; { then }
end; { ShowColor }

{ Shows the main screen }
procedure ShowMainScreen;

begin
  SetColor (15);

  OutTextXY (ColorOffsetX, ColorOffsetY - 10, 'Color in use:');

  OutTextXY (0, 300, 'S          : save picture');
  OutTextXY (0, 310, 'Esc        : end program');
  OutTextXY (0, 320, 'Space      : put new pixel');
  OutTextXY (0, 330, 'Cursorkeys : move the cursor');
  OutTextXY (0, 340, '+ and -    : change the color in use');
end; { ShowMainScreen }

{ Shows a new pixel at the given coord. in the given color }
procedure ShowPixel (X, Y, Color : byte);

begin
  SetFillStyle (1, Color);
  Bar (X * SingleElemSize, Y * SingleElemSize,
       (X + 1) * SingleElemSize - 1, (Y + 1) * SingleElemSize - 1);
  PutPixel (X + SmallOffsetX - 1, Y + SmallOffsetY - 1, Color);
end; { ShowPixel }

{ Restores the large picture from the small one }
procedure GetPicture;

var
  X,
  Y : byte;

begin
  { And put the picture in the array-grid }
  for X := 1 to StoneSize do begin
    for Y := 1 to StoneSize do begin
      Picture [X, Y] := GetPixel (X + SmallOffsetX - 1, Y + SmallOffsetY - 1);
    end; { for }
  end; { for }
end; { GetPicture }






{ File part -----------------------------------------------------------------}

var
  DataFile : file of byte;
  FileName : string;

{ Sets the file name to the one from the command line }
procedure SetFileName;

var
  CmdLine : PChar;

begin
  { Get the file name to use }
  GetMem (CmdLine, 255);
  GetArgStr (CmdLine, 1, 250);
  if CmdLine [0] = #00 then begin
    { No name given -> tell so }
    WriteLn ('No file name specified.');
    WriteLn ('Have to give name of file to process.');
    Halt (1);
  end; { then }
  FileName := StrPas (CmdLine);
  Assign (DataFile, FileName);
end; { SetFileName }

{ Increases the given pointer }
procedure IncPtr (var PictPtr : TPictPtr);

var
  SegPtr,
  OfsPtr : word;

begin
  SegPtr := Seg (PictPtr^);
  OfsPtr := Ofs (PictPtr^);
  Inc (OfsPtr);
  if OfsPtr = 0 then begin
    Inc (SegPtr);
  end; { then }
  PictPtr := Ptr (SegPtr, OfsPtr);
end; { IncPtr }


{ Saves the picture to disk }
procedure SavePicture;

var
  PictElem   : TPictPtr;
  ElemNr     : word;
  X,
  Y          : byte;

begin
  { Put the picture in memory as bitmap }
  GetImage (SmallOffsetX, SmallOffsetY,
            SmallOffsetX + StoneSize - 1, SmallOffsetY + StoneSize - 1,
            PictureSmall^);

  { And start saving it to disk }
  Rewrite (DataFile);
  PictElem := PictureSmall;
  for ElemNr := 1 to PictSize do begin
    Write (DataFile, byte (PictElem^));
    IncPtr (PictElem);
  end;
  Close (DataFile);
end; { SavePicture }

{ Makes a new (standard) data file }
procedure MakeNewFile;

begin
  SavePicture;
end; { MakeNewFile }

{$I-}

{ FileExists : looks whether the given file exists or not }
function FileExists : boolean;

var
  TryFile : text;

begin
  Assign (TryFile, FileName);
  Reset (TryFile);
  FileExists := (IOResult = 0);
end; { FileExists }

{$I+}

{ Reads the picture from file and puts it in memory }
procedure ReadPicture;

var
  X,
  Y,
  PictElemNr : word;
  PictElem   : TPictPtr;

begin
  { Set the file for reading }
  Reset (DataFile);

  { Read the picture from file }
  PictElem := PictureSmall;
  for PictElemNr := 1 to PictSize do begin
    Read (DataFile, byte (PictElem^));
    IncPtr (PictElem);
  end; { for }
  Close (DataFile);
end; { ReadPicture }








{ Keyboard part -------------------------------------------------------------}

const
  Esc    : char = #27;

  Up    : char = #14;
  Down  : char = #15;
  Left  : char = #16;
  Right : char = #17;

{ GetKey : lets the user enter a key on the keyboard }
function GetKey : char;

var
  InChar : char;

  { LowerChar : turns the given char to lower-case }
  function LowerChar (InChar : char) : char;

  begin
    if (InChar >= 'A') and (InChar <= 'Z') then begin
      LowerChar := char (byte (InChar) + 32);
    end { then }
    else begin
      LowerChar := InChar;
    end; { else }
  end; { LowerChar }

begin
  repeat until KeyPressed;
  InChar := ReadKey;
  if InChar = #00 then begin
    InChar := ReadKey;
    case InChar of
      #72 : InChar := Up;
      #75 : InChar := Left;
      #77 : InChar := Right;
      #80 : InChar := Down;
    end; { case }
  end; { then }

  GetKey := LowerChar (InChar);
end; { GetKey }

{ GetSelectedKey : enables the user to enter one of the selected keys and
                   gives back this key }
function GetSelectedKey (Selection : string) : char;

var
  InChar     : char;
  OptionNr   : byte;
  RightInput : boolean;

begin
  RightInput := false;
  repeat
    InChar := GetKey;
    for OptionNr := 1 to Length (Selection) do begin
      if InChar = Selection [OptionNr] then begin
        RightInput := true;
      end; { then }
    end; { for }
  until RightInput;

  GetSelectedKey := InChar;
end; { GetSelectedKey }








{ Cursor and user modification part -----------------------------------------}

var
  PrevX,
  PrevY,
  X,
  Y,
  Color : byte;

{ Puts a new pixel of the specified color in the picture }
procedure PutNewPixel;

begin
  Picture [X, Y] := Color;
  ShowPixel (X, Y, Color);
  ShowCursor (X, Y, PrevX, PrevY);
end; { PutNewPixel }

{ Updates the cursor place }
procedure UpdateCursor;

begin
  PrevX := X;
  PrevY := Y;
end; { UpdateCursor }

{ Moves the cursor up }
procedure MoveUp;

begin
  if Y > 1 then begin
    Dec (Y);
    ShowCursor (X, Y, PrevX, PrevY);
    UpdateCursor;
  end; { then }
end; { MoveUp }

{ Moves the cursor down }
procedure MoveDown;

begin
  if Y < StoneSize then begin
    Inc (Y);
    ShowCursor (X, Y, PrevX, PrevY);
    UpdateCursor;
  end; { then }
end; { MoveDown }

{ Moves the cursor left }
procedure MoveLeft;

begin
  if X > 1 then begin
    Dec (X);
    ShowCursor (X, Y, PrevX, PrevY);
    UpdateCursor;
  end; { then }
end; { MoveLeft }

{ Moves the cursor right }
procedure MoveRight;

begin
  if X < StoneSize then begin
    Inc (X);
    ShowCursor (X, Y, PrevX, PrevY);
    UpdateCursor;
  end; { then }
end; { MoveRight }








{ Main program module -------------------------------------------------------}

begin
  { Reserve memory for the small picture }
  GetMem (PictureSmall, PictSize);

  { Set the file name and file to use }
  SetFileName;

  { Look if the file exists }
  if not FileExists then begin
    WriteLn ('File ''', FileName, ''' doesn''t exist.');
    WriteLn ('Make this file? (Y/N)');
    UserChoice := GetSelectedKey ('yn');
    if UserChoice = 'n' then begin
      Halt (2);
    end; { then }
    InitializeScreen;
    MakeNewFile;
  end; { then }

  { Initialize the screen }
  InitializeScreen;
  PrevX := 1;
  PrevY := 1;
  X     := 1;
  Y     := 1;
  Color := 0;

  { And read the picture }
  ReadPicture;

  { Show the main screen }
  ShowMainScreen;

  { And show the rest }
  ShowPicture;
  GetPicture;
  ShowPicture;
  ShowColor (Color);
  ShowCursor (X, Y, PrevX, PrevY);

  Finished := false;
  Changed := false;
  repeat
    { Wait for user approval }
    UserChoice := GetSelectedKey ('s =-+' + Esc + Up + Down + Left + Right);

    if UserChoice = Esc then begin
      Finished := true;
    end { then }
    else if UserChoice = 's' then begin
      SavePicture;
      Changed := false;
    end { else then }
    else if UserChoice = ' ' then begin
      PutNewPixel;
      Changed := true;
    end { else then }
    else if UserChoice = Up then begin
      MoveUp;
    end { else then }
    else if UserChoice = Down then begin
      MoveDown;
    end { else then }
    else if UserChoice = Left then begin
      MoveLeft;
    end { else then }
    else if UserChoice = Right then begin
      MoveRight;
    end { else then }
    else if UserChoice = '-' then begin
      Color := (Color - 1) and 15;
      ShowColor (Color);
    end { else then }
    else if (UserChoice = '=') or (UserChoice = '+') then begin
      Color := (Color + 1) and 15;
      ShowColor (Color);
    end; { else then }
  until Finished;

  { Look if user changed picture and didn't save }
  if Changed then begin
    SetColor (15);
    OutTextXY (0, 400, 'Picture has been changed. Save to disk before exit? (Y/N)');
    UserChoice := GetSelectedKey ('yn');
    if UserChoice = 'y' then begin
      { Yes -> do so }
      SavePicture;
    end; { then }
  end; {then }

  RestoreCrtMode;
end.